//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Includes
//
#include "aurora.h"
#include "utility.h"
#include <time.h>
#include "strsafe.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// global variables
//
LPCSTR pszAppName = "Aurora";
HINSTANCE g_hInstance;
HWND g_hParent = NULL;

HWND hTop = NULL;
HWND hBottom = NULL;
HWND hLeft = NULL;
HWND hRight = NULL;

int ScreenX, ScreenY;
POINT p;

COLORREF TColorIn, TColorOut, RColorIn, RColorOut;
COLORREF BColorIn, BColorOut, LColorIn, LColorOut;

BOOL PULSE=FALSE;
BOOL PULSE_RANDOM=FALSE;
BOOL HIDDEN=TRUE;
COLORREF PULSE_COLOR[20];
COLORREF CURRENT_COLOR;
int CURRENT_NUM=0;
int NUM_COLORS=0;
int COLOR_STEP=5;
int UPDATE_TIMER=25;


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Message Handling
//
int lsMessages[] = {
	LM_GETREVID,		// RevID handling for about box
	LM_REFRESH,			// !refresh support
	0
};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// MessageHandlerProc
//
// Main Message Handler Procedure
//
LRESULT CALLBACK MessageHandlerProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		// LiteStep wants to know our version info
		case LM_GETREVID:
		{
			if (wParam == 0 || wParam == 1)
			{
				UINT uLength;
				if (SUCCEEDED(StringCchCopy((char*)lParam, REV_BUFFER, "Aurora 0.4 (MrJukes/ilmcuts)"))
				&&  SUCCEEDED(StringCchLength((char*)lParam, REV_BUFFER, &uLength)))
					return uLength;
			}
			
			// something above failed or we don't support this,
			// make sure we don't return something invalid (does this even make sense?)
			lParam = NULL;
		}
		break;

		case LM_REFRESH:
		{
			LoadSetup();
		}
		break;
		
		case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hWnd, &ps);
			RECT cr;
			HBRUSH hbr;

			if (!HIDDEN)
			{
				GetClientRect(hWnd, &cr);

				if (!PULSE)
				{
					if (hWnd == hTop)
					{
						int fr = (GetRValue(TColorIn) - p.y * GetRValue(TColorIn) / ScreenY) + (p.y * GetRValue(TColorOut) / ScreenY);
						int fg = (GetGValue(TColorIn) - p.y * GetGValue(TColorIn) / ScreenY) + (p.y * GetGValue(TColorOut) / ScreenY);
						int fb = (GetBValue(TColorIn) - p.y * GetBValue(TColorIn) / ScreenY) + (p.y * GetBValue(TColorOut) / ScreenY);
						hbr = CreateSolidBrush(RGB(fr, fg, fb));
					}
					else if (hWnd == hRight)
					{
						int fr = (GetRValue(RColorOut) - p.x * GetRValue(RColorOut) / ScreenX) + (p.x * GetRValue(RColorIn) / ScreenX);
						int fg = (GetGValue(RColorOut) - p.x * GetGValue(RColorOut) / ScreenX) + (p.x * GetGValue(RColorIn) / ScreenX);
						int fb = (GetBValue(RColorOut) - p.x * GetBValue(RColorOut) / ScreenX) + (p.x * GetBValue(RColorIn) / ScreenX);
						hbr = CreateSolidBrush(RGB(fr, fg, fb));
					}
					else if (hWnd == hBottom)
					{
						int fr = (GetRValue(BColorOut) - p.y * GetRValue(BColorOut) / ScreenY) + (p.y * GetRValue(BColorIn) / ScreenY);
						int fg = (GetGValue(BColorOut) - p.y * GetGValue(BColorOut) / ScreenY) + (p.y * GetGValue(BColorIn) / ScreenY);
						int fb = (GetBValue(BColorOut) - p.y * GetBValue(BColorOut) / ScreenY) + (p.y * GetBValue(BColorIn) / ScreenY);
						hbr = CreateSolidBrush(RGB(fr, fg, fb));
					}
					else if (hWnd == hLeft)
					{
						int fr = (GetRValue(LColorIn) - p.x * GetRValue(LColorIn) / ScreenX) + (p.x * GetRValue(LColorOut) / ScreenX);
						int fg = (GetGValue(LColorIn) - p.x * GetGValue(LColorIn) / ScreenX) + (p.x * GetGValue(LColorOut) / ScreenX);
						int fb = (GetBValue(LColorIn) - p.x * GetBValue(LColorIn) / ScreenX) + (p.x * GetBValue(LColorOut) / ScreenX);
						hbr = CreateSolidBrush(RGB(fr, fg, fb));
					}
				}
				else
				{
					hbr = CreateSolidBrush(CURRENT_COLOR);
				}

				FillRect(hdc, &cr, hbr);
			}

			EndPaint(hWnd, &ps);
			DeleteDC(hdc);
			DeleteObject(hbr);
		}
		break;

		case WM_TIMER:
		{
			switch ((int)wParam)
			{
				case 0:
				{
					POINT tp;
					GetCursorPos(&tp);

					if (tp.x == p.x && tp.y == p.y) break;

					p.x = tp.x;
					p.y = tp.y;

					repaint();
				}
				break;

				case 1:
				{
					static BOOL BUSY=FALSE;

					if (!BUSY)
					{
						int cr, cg, cb;
						int nr, ng, nb;

						BUSY=TRUE;

						if (!PULSE)
						{
							PULSE=TRUE;
							CURRENT_NUM=0;
							CURRENT_COLOR = PULSE_COLOR[0];
						}

						if (CURRENT_NUM == NUM_COLORS) 
						{
							CURRENT_NUM = 0;
							if (PULSE_RANDOM) 
							{
								PULSE_COLOR[0] = RGB(rand()%255, rand()%255, rand()%255);
								PULSE_COLOR[1] = RGB(rand()%255, rand()%255, rand()%255);
							}
						}

						cr = GetRValue(CURRENT_COLOR);
						cg = GetGValue(CURRENT_COLOR);
						cb = GetBValue(CURRENT_COLOR);

						if (CURRENT_NUM == NUM_COLORS-1)
						{
							nr = GetRValue(PULSE_COLOR[0]);
							ng = GetGValue(PULSE_COLOR[0]);
							nb = GetBValue(PULSE_COLOR[0]);
						}
						else
						{
							nr = GetRValue(PULSE_COLOR[CURRENT_NUM+1]);
							ng = GetGValue(PULSE_COLOR[CURRENT_NUM+1]);
							nb = GetBValue(PULSE_COLOR[CURRENT_NUM+1]);
						}

						if (cr == nr && cg == ng && cb == nb) 
						{ 
							CURRENT_NUM++; 
							BUSY=FALSE; 
							break;
						}

						if (abs(cr - nr) <= COLOR_STEP) cr = nr;
						else if (cr < nr) cr += COLOR_STEP;
						else if (cr > nr) cr -= COLOR_STEP;

						if (abs(cg - ng) <= COLOR_STEP) cg = ng;
						if (cg < ng) cg += COLOR_STEP;
						else if (cg > ng) cg -= COLOR_STEP;

						if (abs(cb - nb) <= COLOR_STEP) cb = nb;
						if (cb < nb) cb += COLOR_STEP;
						else if (cb > nb) cb -= COLOR_STEP;

						CURRENT_COLOR = RGB(cr, cg, cb);
						
						repaint();

						BUSY=FALSE;
					}
				}
				break;
			}
		}
		break;
	}

	return DefWindowProc(hWnd,uMsg, wParam,lParam);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// initModuleEx
//
// Module Initialization
//
int initModuleEx(HWND hParent, HINSTANCE /* hInstance */, LPCSTR /* szLSpath */)
{
	if (IsWindow(hParent))
	{
		g_hParent = hParent;
	}
	else
	{
		g_hParent = GetLitestepWnd();
		
		if (!IsWindow(g_hParent))
		{
			ErrorMessage("Error obtaining parent window.");
			return 1;
		}
	}
	
	// Register class for Main Message Handler
	WNDCLASS wc;
	
	ZeroMemory(&wc, sizeof(WNDCLASS));
	
	wc.style = CS_NOCLOSE;
	wc.lpfnWndProc = MessageHandlerProc;
	wc.hInstance = g_hInstance;
	wc.lpszClassName = pszAppName;
	
	if (!RegisterClass(&wc)) 
	{
		ErrorMessage("Error registering window class");
		return 1;
	}
    
	// load step.rc settings
	LoadSetup();

	// ugly hack... we're abusing the "top" window for LM_* - add a hMain next time ;)
	PostMessage(GetLitestepWnd(), LM_REGISTERMESSAGE, (WPARAM)hTop, (LPARAM)lsMessages);

	// register !bang commands
	AddBangCommand("!AuroraHide", BangHide);
	AddBangCommand("!AuroraShow", BangShow);
	AddBangCommand("!AuroraToggle", BangToggle);
	AddBangCommand("!AuroraPulse", BangPulse);
	AddBangCommand("!AuroraStopPulse", BangStopPulse);
	AddBangCommand("!AuroraInvert", BangInvert);

	return 0;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// quitModule
//
// module cleanup
//
void quitModule(HINSTANCE /* hInstance */)
{
	SendMessage(GetLitestepWnd(), LM_UNREGISTERMESSAGE, (WPARAM)hTop, (LPARAM)lsMessages);

	RemoveBangCommand("!AuroraHide");
	RemoveBangCommand("!AuroraShow");
	RemoveBangCommand("!AuroraToggle");
	RemoveBangCommand("!AuroraPulse");
	RemoveBangCommand("!AuroraStopPulse");
	RemoveBangCommand("!AuroraInvert");
	
	KillTimer(hTop, 0);
	KillTimer(hTop, 1);
	DestroyWindow(hTop);
	DestroyWindow(hRight);
	DestroyWindow(hBottom);
	DestroyWindow(hLeft);
	UnregisterClass(pszAppName, g_hInstance);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// LoadSetup
//
// read modules settings from step.rc
//
void LoadSetup()
{
	ScreenX = GetSystemMetrics(SM_CXSCREEN);
	ScreenY = GetSystemMetrics(SM_CYSCREEN);

	COLOR_STEP = GetRCInt("AuroraColorStep", 5);
	UPDATE_TIMER = GetRCInt("AuroraTimer", 25);

	int X = GetRCInt("AuroraX", 0);
	int Y = GetRCInt("AuroraY", 0);
	UINT W = BOUND(GetRCInt("AuroraWidth", ScreenX), 0, ScreenX);
	UINT H = BOUND(GetRCInt("AuroraHeight", ScreenY), 0, ScreenY);

	ConfigureAuroraWnd(hTop, X, Y, W, GetRCInt("AuroraTopWidth", 2));
	ConfigureAuroraWnd(hRight, X+W-GetRCInt("AuroraRightWidth", 2), Y, GetRCInt("AuroraRightWidth", 2), H);
	ConfigureAuroraWnd(hBottom, X, Y+H-GetRCInt("AuroraBottomWidth", 2), W, GetRCInt("AuroraBottomWidth", 2));
	ConfigureAuroraWnd(hLeft, X, Y, GetRCInt("AuroraLeftWidth", 2), H);

	BOOL _bHidden = GetRCBool("AuroraStartHidden", TRUE);
	
	if (HIDDEN != _bHidden)
	{
		ShowWindowEx(hTop, SW_TOGGLE);
		ShowWindowEx(hRight, SW_TOGGLE);
		ShowWindowEx(hBottom, SW_TOGGLE);
		ShowWindowEx(hLeft, SW_TOGGLE);
		HIDDEN = _bHidden;
	}
	
	if (GetRCBool("AuroraStartPulsing", TRUE))
	{
		SAFE_CHAR(szTemp, MAX_BANGARGS);
		GetRCLine("AuroraPulse", szTemp, MAX_BANGARGS, "FFFFFF 000000");
		BangPulse(hTop, szTemp);
	}
	else
	{
		SetTimer(hTop, 0, UPDATE_TIMER, NULL);
	}
	
	TColorIn = GetRCColor("AuroraTopColorIn", 0xAA0000);
	TColorOut = GetRCColor("AuroraTopColorOut", 0x0000AA);
	
	RColorIn = GetRCColor("AuroraRightColorIn", 0xAA0000);
	RColorOut = GetRCColor("AuroraRightColorOut", 0x0000AA);

	BColorIn = GetRCColor("AuroraBottomColorIn", 0xAA0000);
	BColorOut = GetRCColor("AuroraBottomColorOut", 0x0000AA);

	LColorIn = GetRCColor("AuroraLeftColorIn", 0xAA0000);
	LColorOut = GetRCColor("AuroraLeftColorOut", 0x0000AA);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// DllMain
//
// DLL entry point
//
BOOL APIENTRY DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /* pvReserved */)
{
	// supposedly used to get rid of CRT...
	if(dwReason == DLL_PROCESS_ATTACH)
	{
		DisableThreadLibraryCalls(hInstance);
	}
	
	// save our instance for future reference
	g_hInstance = hInstance;
	
	return TRUE;
}


void BangPulse(HWND /* hCaller */, LPCSTR pszArgs)
{
	SAFE_CHAR(szBuffer, MAX_BANGARGS);
	char *endstr;
	char *p;

	StringCchCopy(szBuffer, MAX_BANGARGS, pszArgs);

	p = strtok(szBuffer, " ");

	NUM_COLORS=0;

	while (p)
	{
		if(!_strnicmp( p, "0x", 2 )) p += 2;
		else if (!_strnicmp(p, "!Random", 7 )) 
		{
			PULSE_RANDOM=TRUE;
			NUM_COLORS=2;
			srand(time(NULL));
			PULSE_COLOR[0] = RGB(rand()%255, rand()%255, rand()%255);
			PULSE_COLOR[1] = RGB(rand()%255, rand()%255, rand()%255);
			break;
		}

		PULSE_COLOR[NUM_COLORS++] = strtol( p, &endstr, 16 );
		p = strtok(NULL, " ");
	}

	KillTimer(hTop, 0);
	SetTimer(hTop, 1, UPDATE_TIMER, NULL);
}

void BangStopPulse(HWND /* hCaller */, LPCSTR /* pszArgs */)
{
	KillTimer(hTop, 1);
	PULSE=FALSE;
	SetTimer(hTop, 0, UPDATE_TIMER, NULL);

	repaint();
}

void BangHide(HWND /* hCaller */, LPCSTR /* pszArgs */)
{
	HIDDEN=TRUE;
	ShowWindowEx(hTop, SW_HIDE);
	ShowWindowEx(hRight, SW_HIDE);
	ShowWindowEx(hBottom, SW_HIDE);
	ShowWindowEx(hLeft, SW_HIDE);
	KillTimer(hTop, 0);
	KillTimer(hTop, 1);
}

void BangShow(HWND /*hCaller */, LPCSTR /* pszArgs */)
{
	HIDDEN=FALSE;
	ShowWindowEx(hTop, SW_SHOW);
	ShowWindowEx(hRight, SW_SHOW);
	ShowWindowEx(hBottom, SW_SHOW);
	ShowWindowEx(hLeft, SW_SHOW);
	if (PULSE) SetTimer(hTop, 1, UPDATE_TIMER, NULL);
	else SetTimer(hTop, 0, UPDATE_TIMER, NULL);
}

void BangToggle(HWND /* hCaller */, LPCSTR /* pszArgs */)
{
	ShowWindowEx(hTop, SW_TOGGLE);
	ShowWindowEx(hRight, SW_TOGGLE);
	ShowWindowEx(hBottom, SW_TOGGLE);
	ShowWindowEx(hLeft, SW_TOGGLE);

	HIDDEN = !HIDDEN;
}

void BangInvert(HWND /* hCaller */, LPCSTR /* pszArgs */)
{
	SWAP(TColorIn, TColorOut);
	SWAP(RColorIn, RColorOut);
	SWAP(BColorIn, BColorOut);
	SWAP(LColorIn, LColorOut);
	
	repaint();
}

void repaint()
{
	InvalidateRect(hTop, NULL, TRUE);
	InvalidateRect(hRight, NULL, TRUE);
	InvalidateRect(hBottom, NULL, TRUE);
	InvalidateRect(hLeft, NULL, TRUE);
}